@@ -500,11 +500,21 @@ def find_relevant_segment(question, segments):
 import json
 import random
 
-GREETINGS_INPUT = ["hi", "hello", "hey", "greetings", "good day", "good morning", "good afternoon", "good evening"]
-# Responses the system can give to greetings
-GREETINGS_RESPONSE = ["Hello!", "Hi there!", "Hello! How can I assist you today?","Good Morning", "Good Afternoon", "Good Evening"]
-
+# GREETINGS_INPUT = ["hi", "hello", "hey", "greetings", "good day", "good morning", "good afternoon", "good evening"]
+# # Responses the system can give to greetings
+# GREETINGS_RESPONSE = ["Hello!", "Hi there!", "Hello! How can I assist you today?","Good Morning", "Good Afternoon", "Good Evening"]
 
+GREETINGS_MAP = {
+    "hi": "Hi there!",
+    "hello": "Hello! How can I assist you today?",
+    "hey": "Hey! What can I do for you?",
+    "greetings": "Greetings! How may I help?",
+    "good day": "Good day to you!",
+    "good morning": "Good morning! Hope you're having a great day!",
+    "good afternoon": "Good afternoon! How can I assist?",
+    "good evening": "Good evening! What do you need help with?",
+    "how are you": "I'm just a bot, but thank you for asking! How can I assist you today?"
+}
 
 @csrf_exempt
 def chat(request, user_name, file_id):
@@ -544,17 +554,27 @@ def chat(request, user_name, file_id):
             return HttpResponseBadRequest('Missing or empty "question" parameter in the request.')
         
         
-        if question in GREETINGS_INPUT:
-            random_greeting = random.choice(GREETINGS_RESPONSE)  # Select a random greeting response
-            question_asked_time = timezone.localtime(timezone.now())
+        # if question in GREETINGS_INPUT:
+        #     random_greeting = random.choice(GREETINGS_RESPONSE)  # Select a random greeting response
+        #     question_asked_time = timezone.localtime(timezone.now())
+        #     return JsonResponse({
+        #         'answer': random_greeting,
+        #         'source': 'system',
+        #         'session_id': session_id,
+        #         'question_asked_time': question_asked_time.strftime("%d-%m-%Y %H:%M:%S")
+                
+                
+        #     })
+
+
+
+        if question in GREETINGS_MAP:
+            response_message = GREETINGS_MAP[question]
             return JsonResponse({
-                'answer': random_greeting,
-                'source': 'system',
-                'session_id': session_id,
-                'question_asked_time': question_asked_time.strftime("%d-%m-%Y %H:%M:%S")
-                
-                
+                'answer': response_message,
+                'source': 'system'
             })
+        
 
 
         question_asked_time = timezone.localtime(timezone.now())
@@ -635,15 +655,7 @@ def chat(request, user_name, file_id):
 
         text_segments = split_text(document_text)
 
-        def query_sagemaker_endpoint(endpoint_name, payload):
-            runtime = boto3.client('sagemaker-runtime', region_name='ap-south-1')
-            response = runtime.invoke_endpoint(
-                EndpointName=endpoint_name,
-                ContentType='application/json',
-                Body=json.dumps(payload)
-            )
-            response_body = response['Body'].read().decode('utf-8')
-            return json.loads(response_body)
+        
 
         from transformers import GPT2Tokenizer
 
@@ -655,73 +667,94 @@ def chat(request, user_name, file_id):
             return len(tokens)
 
         def clean_response(response_text, stop_sequences=["<end>", "</s>", "</n>"]):
-            # Extract the answer after 'assistant:' and stop at the first stop sequence found
-            answer_start = response_text.find('assistant:') + len('assistant:')
-            response_text = response_text[answer_start:].strip()
-
             # Find the earliest occurrence of any stop sequence
-            first_stop_index = min((response_text.find(seq) for seq in stop_sequences if seq in response_text), default=len(response_text))
-            answer = response_text[:first_stop_index].strip()
+            first_stop_index = min((response_text.find(seq) if seq in response_text else len(response_text) for seq in stop_sequences))
+            
+            # Truncate the response at the first stop sequence
+            response_text = response_text[:first_stop_index].strip()
 
+            # Remove any known unwanted tokens
             unwanted_tokens = ["ASSISTANT:", "assistant:", "user:"]
             for token in unwanted_tokens:
-                answer = answer.replace(token, "").strip()
+                response_text = response_text.replace(token, "").strip()
 
-            # Ensure the response does not end mid-sentence
-            # if answer and answer[-1] not in '.!?':
-            #     last_punctuation_index = max(answer.rfind(char) for char in '.!?')
-            #     if last_punctuation_index != -1:
-            #         answer = answer[:last_punctuation_index + 1]
-            #     else:
-            #         answer += "."
+            return response_text
 
-            return answer
+        from transformers import GPT2Tokenizer
 
-        def get_response(file_id, relevant_text, question, stop_sequences=["<end>", "</s>", "</n>"], max_attempts=3):
+# Initialize the Bedrock client globally
+        bedrock_client = boto3.client(service_name='bedrock-runtime')
+
+        # Initialize the tokenizer
+        tokenizer = GPT2Tokenizer.from_pretrained('gpt2')
+
+       
+
+        def initialize_bedrock_client():
+            # Create a Bedrock client
+            return boto3.client(service_name='bedrock-runtime')
+        
+        def truncate_context(context, max_tokens):
+            tokens = tokenizer.encode(context, add_special_tokens=False)
+            if len(tokens) > max_tokens:
+                truncated_tokens = tokens[:max_tokens]
+                return tokenizer.decode(truncated_tokens)
+            return context
+
+        def get_response(file_id, relevant_text,context, question, stop_sequences=["<end>", "</s>", "</n>"], max_attempts=3):
             print(relevant_text)
+            max_context_tokens = 8192 - 1000  # Reserve tokens for the user input and response
+            truncated_context = truncate_context(context, max_context_tokens)
+            print(truncated_context)
             messages = [
                 {"role": "system", "content": "You are a helpful assistant. Provide a brief, concise answer using only the uploaded document's content."},
                 {"role": "system", "content": "You are a helpful assistant. Answer the following question based solely on the uploaded document content. Be brief and terminate your response with '<end>'."},
                 {"role": "system", "content": "You are a helpful assistant. Always add a full stop at the end of the response"},
+                {"role": "system", "content": "If the question is related to politicians ,film industry, cricketers,singers, languages and is outside the scope of document, respond with 'I don't know' and terminate your response with '<end>'."},
                 {"role": "user", "content": f"Document details (File ID: {file_id}):"},
-                {"role": "user", "content": relevant_text},
+                {"role": "user", "content": truncated_context},
+                
                 {"role": "user", "content": question}
             ]
-
             full_prompt = "\n".join([f"{msg['role']}: {msg['content']}" for msg in messages]) + "\nassistant:"
 
-            payload = {
-                "inputs": full_prompt,
-                "parameters": {
-                    "top_k": 1,
-                    "top_p": 0.9,
-                    "temperature": 0.7,
-                    "length_penalty": 1,  # Encouraging balanced responses
-                    "max_new_tokens": 1024,   # Increase max tokens to ensure enough length
-                    "stop_sequences": stop_sequences  # Custom stop sequences to limit response
-                }
-            }
+            body = json.dumps({
+                "prompt": full_prompt,
+                "temperature": 0.1,
+                "top_p": 0.9
+            })
 
+            modelId = 'meta.llama3-8b-instruct-v1:0'
+            accept = 'application/json'
+            contentType = 'application/json'
+
+            # Invoke the model with the input body
+            response = bedrock_client.invoke_model(body=body, modelId=modelId, accept=accept, contentType=contentType)
+
+            # Read the StreamingBody object as a string
+            response_body_str = response['body'].read().decode('utf-8')
             try:
                 response_text = ""
-                for attempt in range(max_attempts):
-                    response = query_sagemaker_endpoint("jumpstart-dft-gradientai-llama-3-8b-20240529-065312", payload)
-                    generated_text = response[0]['generated_text']
-                    print("Generated text:", generated_text)  # Debugging output to inspect generated text
-
-                    response_text += generated_text
-                    # Check if the response contains a stop sequence
-                    if any(seq in generated_text for seq in stop_sequences):
-                        break
+                response_body = json.loads(response_body_str)
+                # print("Debug: Response Body:", response_body)  # Debugging line
                 
+                # Extract and return the generation text if available
+                generation = response_body.get('generation')
+                print(f"Generation Text :{generation}")
+                response_text += generation
                 clean_resp = clean_response(response_text, stop_sequences)
                 token_count = count_tokens(clean_resp)
                 print(f"Token count of the response: {token_count}")
+                # return generation if generation else "Sorry, I didn't understand that."
+                return clean_response(generation) if generation else "Sorry, I didn't understand that."
+            except json.JSONDecodeError as e:
+                print(f"Error decoding JSON: {str(e)}")
+                print("Debug: Response Body String:", response_body_str)  # Debugging line
+                return "Sorry, there was an error processing the response."
+            
+        bedrock_client = initialize_bedrock_client()
 
-                return clean_resp
-            except Exception as e:
-                print(f"Error occurred: {e}")
-                return "An error occurred while processing your request."
+            
         
 
             
@@ -729,7 +762,7 @@ def chat(request, user_name, file_id):
         relevant_segment = find_relevant_segment(question, text_segments)
         # prompt = f"Context: {relevant_segment}\n\nQuestion: {question}\n### Answer:"
         # Assuming 'relevant_segment' is the context and 'question' is the actual question being asked.
-        response = get_response(file_id, relevant_segment, question)
+        response = get_response(file_id,bedrock_client, relevant_segment, question)
 
 
         print(f"Chatbot: {response}")
@@ -901,16 +934,6 @@ def count_negative_feedback(question):
 import json
 from django.db.models import F
 from django.db.models import Max
-
-
-
-
-
-
-
-
-
-
 from django.contrib.sessions.models import Session
 from django.contrib.sessions.backends.db import SessionStore
 
